@@ -1,5 +1,5 @@
 /* xgettext Emacs Lisp backend.
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002-2003 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2002.
 
    This program is free software; you can redistribute it and/or modify
@@ -21,11 +21,13 @@
   { "el",        "EmacsLisp"     },					\
 
 #define SCANNERS_ELISP \
-  { "EmacsLisp",  extract_elisp, &formatstring_elisp },			\
+  { "EmacsLisp",	extract_elisp,					\
+			&flag_table_elisp, &formatstring_elisp, NULL },	\
 
 /* Scan an Emacs Lisp file and add its translatable strings to mdlp.  */
 extern void extract_elisp (FILE *fp, const char *real_filename,
 			   const char *logical_filename,
+			   flag_context_list_table_ty *flag_table,
 			   msgdomain_list_ty *mdlp);
 
 
@@ -33,3 +35,5 @@ extern void extract_elisp (FILE *fp, const char *real_filename,
 
 extern void x_elisp_extract_all (void);
 extern void x_elisp_keyword (const char *name);
+
+extern void init_flag_table_elisp (void);
