@@ -1,5 +1,5 @@
 /* xgettext Lisp backend.
-   Copyright (C) 2001-2002 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2005 Free Software Foundation, Inc.
 
    This file was written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
@@ -28,10 +28,10 @@
 #include <string.h>
 
 #include "message.h"
-#include "x-lisp.h"
 #include "xgettext.h"
+#include "x-lisp.h"
 #include "error.h"
-#include "xmalloc.h"
+#include "xalloc.h"
 #include "exit.h"
 #include "hash.h"
 #include "gettext.h"
@@ -178,6 +178,16 @@ init_keywords ()
     }
 }
 
+void
+init_flag_table_lisp ()
+{
+  xgettext_record_flag ("gettext:1:pass-lisp-format");
+  xgettext_record_flag ("ngettext:1:pass-lisp-format");
+  xgettext_record_flag ("ngettext:2:pass-lisp-format");
+  xgettext_record_flag ("gettext-noop:1:pass-lisp-format");
+  xgettext_record_flag ("format:2:lisp-format");
+}
+
 
 /* ======================== Reading of characters.  ======================== */
 
@@ -580,7 +590,7 @@ is_potential_number (const struct token *tp, int *basep)
   return true;
 }
 
-/* A number is one integer, ratio, float.  Each has a particular syntax.
+/* A number is one of integer, ratio, float.  Each has a particular syntax.
    See CLHS 2.3.1 "Numbers as Tokens".
    But note a mistake: The exponent rule should read:
        exponent ::= exponent-marker [sign] {decimal-digit}+
@@ -831,7 +841,7 @@ comment_add (int c)
 {
   if (buflen >= bufmax)
     {
-      bufmax += 100;
+      bufmax = 2 * bufmax + 10;
       buffer = xrealloc (buffer, bufmax);
     }
   buffer[buflen++] = c;
@@ -846,7 +856,7 @@ comment_line_end (size_t chars_to_remove)
     --buflen;
   if (chars_to_remove == 0 && buflen >= bufmax)
     {
-      bufmax += 100;
+      bufmax = 2 * bufmax + 10;
       buffer = xrealloc (buffer, bufmax);
     }
   buffer[buflen] = '\0';
@@ -919,9 +929,12 @@ string_of_object (const struct object *op)
   return str;
 }
 
+/* Context lookup table.  */
+static flag_context_list_table_ty *flag_context_list_table;
+
 /* Read the next object.  */
 static void
-read_object (struct object *op)
+read_object (struct object *op, flag_context_ty outer_context)
 {
   for (;;)
     {
@@ -998,6 +1011,7 @@ read_object (struct object *op)
 	    case '(':
 	      {
 		int arg = 0;		/* Current argument number.  */
+		flag_context_list_iterator_ty context_iter;
 		int argnum1 = 0;	/* First string position.  */
 		int argnum2 = 0;	/* Plural string position.  */
 		message_ty *plural_mp = NULL;	/* Remember the msgid.  */
@@ -1005,8 +1019,17 @@ read_object (struct object *op)
 		for (;; arg++)
 		  {
 		    struct object inner;
+		    flag_context_ty inner_context;
 
-		    read_object (&inner);
+		    if (arg == 0)
+		      inner_context = null_context;
+		    else
+		      inner_context =
+			inherited_context (outer_context,
+					   flag_context_list_iterator_advance (
+					     &context_iter));
+
+		    read_object (&inner, inner_context);
 
 		    /* Recognize end of list.  */
 		    if (inner.type == t_close)
@@ -1025,68 +1048,74 @@ read_object (struct object *op)
 		    if (inner.type == t_eof)
 		      break;
 
-		    /* No need to bother if we extract all strings anyway.  */
-		    if (!extract_all)
+		    if (arg == 0)
 		      {
-			if (arg == 0)
+			/* This is the function position.  */
+			if (inner.type == t_symbol)
 			  {
-			    /* This is the function position.  */
-			    if (inner.type == t_symbol)
+			    char *symbol_name = string_of_object (&inner);
+			    int i;
+			    int prefix_len;
+			    void *keyword_value;
+
+			    /* Omit any package name.  */
+			    i = inner.token->charcount;
+			    while (i > 0
+				   && inner.token->chars[i-1].attribute != a_pack_m)
+			      i--;
+			    prefix_len = i;
+
+			    if (find_entry (&keywords,
+					    symbol_name + prefix_len,
+					    strlen (symbol_name + prefix_len),
+					    &keyword_value)
+				== 0)
 			      {
-				char *symbol_name = string_of_object (&inner);
-				int i;
-				int prefix_len;
-				void *keyword_value;
-
-				/* Omit any package name.  */
-				i = inner.token->charcount;
-				while (i > 0
-				       && inner.token->chars[i-1].attribute != a_pack_m)
-				  i--;
-				prefix_len = i;
-
-				if (find_entry (&keywords,
-						symbol_name + prefix_len,
-						strlen (symbol_name + prefix_len),
-						&keyword_value)
-				    == 0)
-				  {
-				    argnum1 = (int) (long) keyword_value & ((1 << 10) - 1);
-				    argnum2 = (int) (long) keyword_value >> 10;
-				  }
-
-				free (symbol_name);
+				argnum1 = (int) (long) keyword_value & ((1 << 10) - 1);
+				argnum2 = (int) (long) keyword_value >> 10;
 			      }
+
+			    context_iter =
+			      flag_context_list_iterator (
+				flag_context_list_table_lookup (
+				  flag_context_list_table,
+				  symbol_name, strlen (symbol_name)));
+
+			    free (symbol_name);
 			  }
 			else
+			  context_iter = null_context_list_iterator;
+		      }
+		    else
+		      {
+			/* These are the argument positions.
+			   Extract a string if we have reached the right
+			   argument position.  */
+			if (arg == argnum1)
 			  {
-			    /* These are the argument positions.
-			       Extract a string if we have reached the right
-			       argument position.  */
-			    if (arg == argnum1)
+			    if (inner.type == t_string)
 			      {
-				if (inner.type == t_string)
-				  {
-				    lex_pos_ty pos;
-				    message_ty *mp;
+				lex_pos_ty pos;
+				message_ty *mp;
 
-				    pos.file_name = logical_file_name;
-				    pos.line_number = inner.line_number_at_start;
-				    mp = remember_a_message (mlp, string_of_object (&inner), &pos);
-				    if (argnum2 > 0)
-				      plural_mp = mp;
-				  }
+				pos.file_name = logical_file_name;
+				pos.line_number = inner.line_number_at_start;
+				mp = remember_a_message (mlp, string_of_object (&inner),
+							 inner_context, &pos);
+				if (argnum2 > 0)
+				  plural_mp = mp;
 			      }
-			    else if (arg == argnum2)
+			  }
+			else if (arg == argnum2)
+			  {
+			    if (inner.type == t_string && plural_mp != NULL)
 			      {
-				if (inner.type == t_string && plural_mp != NULL)
-				  {
-				    lex_pos_ty pos;
+				lex_pos_ty pos;
 
-				    pos.file_name = logical_file_name;
-				    pos.line_number = inner.line_number_at_start;
-				    remember_a_message_plural (plural_mp, string_of_object (&inner), &pos);
-				  }
+				pos.file_name = logical_file_name;
+				pos.line_number = inner.line_number_at_start;
+				remember_a_message_plural (plural_mp, string_of_object (&inner),
+							   inner_context, &pos);
 			      }
 			  }
 		      }
@@ -1120,7 +1149,7 @@ read_object (struct object *op)
 	      {
 		struct object inner;
 
-		read_object (&inner);
+		read_object (&inner, null_context);
 
 		/* Dots and EOF are not allowed here.  But be tolerant.  */
 
@@ -1145,7 +1174,11 @@ read_object (struct object *op)
 		    if (c != ';')
 		      all_semicolons = false;
 		    if (!all_semicolons)
-		      comment_add (c);
+		      {
+			/* We skip all leading white space, but not EOLs.  */
+			if (!(buflen == 0 && (c == ' ' || c == '\t')))
+			  comment_add (c);
+		      }
 		  }
 		comment_line_end (0);
 		continue;
@@ -1182,7 +1215,8 @@ read_object (struct object *op)
 
 		    pos.file_name = logical_file_name;
 		    pos.line_number = op->line_number_at_start;
-		    remember_a_message (mlp, string_of_object (op), &pos);
+		    remember_a_message (mlp, string_of_object (op),
+					null_context, &pos);
 		  }
 		last_non_comment_line = line_number;
 		return;
@@ -1222,7 +1256,7 @@ read_object (struct object *op)
 		  case 'S': case 's':
 		    {
 		      struct object inner;
-		      read_object (&inner);
+		      read_object (&inner, null_context);
 		      /* Dots and EOF are not allowed here.
 			 But be tolerant.  */
 		      free_object (&inner);
@@ -1340,7 +1374,7 @@ read_object (struct object *op)
 		    /* Simply assume every feature expression is true.  */
 		    {
 		      struct object inner;
-		      read_object (&inner);
+		      read_object (&inner, null_context);
 		      /* Dots and EOF are not allowed here.
 			 But be tolerant.  */
 		      free_object (&inner);
@@ -1372,6 +1406,7 @@ read_object (struct object *op)
 void
 extract_lisp (FILE *f,
 	      const char *real_filename, const char *logical_filename,
+	      flag_context_list_table_ty *flag_table,
 	      msgdomain_list_ty *mdlp)
 {
   mlp = mdlp->item[0]->messages;
@@ -1384,6 +1419,8 @@ extract_lisp (FILE *f,
   last_comment_line = -1;
   last_non_comment_line = -1;
 
+  flag_context_list_table = flag_table;
+
   init_keywords ();
 
   /* Eat tokens until eof is seen.  When read_object returns
@@ -1392,7 +1429,7 @@ extract_lisp (FILE *f,
     {
       struct object toplevel_object;
 
-      read_object (&toplevel_object);
+      read_object (&toplevel_object, null_context);
 
       if (toplevel_object.type == t_eof)
 	break;
