@@ -1,5 +1,5 @@
 /* xgettext RST backend.
-   Copyright (C) 2001-2002 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -21,9 +21,11 @@
   { "rst",    "RST"   },						\
 
 #define SCANNERS_RST \
-  { "RST",        extract_rst, &formatstring_pascal },			\
+  { "RST",		extract_rst,					\
+			NULL, &formatstring_pascal, NULL },		\
 
 /* Scan an RST file and add its translatable strings to mdlp.  */
 extern void extract_rst (FILE *fp, const char *real_filename,
 			 const char *logical_filename,
+			 flag_context_list_table_ty *flag_table,
 			 msgdomain_list_ty *mdlp);
