@@ -1,5 +1,5 @@
 /* Extracts strings from C source file to Uniforum style .po file.
-   Copyright (C) 1995-1998, 2000-2003 Free Software Foundation, Inc.
+   Copyright (C) 1995-1998, 2000-2005 Free Software Foundation, Inc.
    Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, April 1995.
 
    This program is free software; you can redistribute it and/or modify
@@ -44,11 +44,12 @@
 #include "basename.h"
 #include "xerror.h"
 #include "xalloc.h"
+#include "xallocsa.h"
 #include "strstr.h"
 #include "xerror.h"
 #include "exit.h"
 #include "pathname.h"
-#include "strcase.h"
+#include "c-strcase.h"
 #include "stpcpy.h"
 #include "open-po.h"
 #include "read-po-abstract.h"
@@ -76,9 +77,11 @@ extern "C" {
 #include "x-lisp.h"
 #include "x-elisp.h"
 #include "x-librep.h"
+#include "x-scheme.h"
 #include "x-smalltalk.h"
 #include "x-java.h"
 #include "x-properties.h"
+#include "x-csharp.h"
 #include "x-awk.h"
 #include "x-ycp.h"
 #include "x-tcl.h"
@@ -142,7 +145,9 @@ static flag_context_list_table_ty flag_table_python;
 static flag_context_list_table_ty flag_table_lisp;
 static flag_context_list_table_ty flag_table_elisp;
 static flag_context_list_table_ty flag_table_librep;
+static flag_context_list_table_ty flag_table_scheme;
 static flag_context_list_table_ty flag_table_java;
+static flag_context_list_table_ty flag_table_csharp;
 static flag_context_list_table_ty flag_table_awk;
 static flag_context_list_table_ty flag_table_ycp;
 static flag_context_list_table_ty flag_table_tcl;
@@ -296,7 +301,9 @@ main (int argc, char *argv[])
   init_flag_table_lisp ();
   init_flag_table_elisp ();
   init_flag_table_librep ();
+  init_flag_table_scheme ();
   init_flag_table_java ();
+  init_flag_table_csharp ();
   init_flag_table_awk ();
   init_flag_table_ycp ();
   init_flag_table_tcl ();
@@ -317,7 +324,9 @@ main (int argc, char *argv[])
 	x_lisp_extract_all ();
 	x_elisp_extract_all ();
 	x_librep_extract_all ();
+	x_scheme_extract_all ();
 	x_java_extract_all ();
+	x_csharp_extract_all ();
 	x_awk_extract_all ();
 	x_tcl_extract_all ();
 	x_perl_extract_all ();
@@ -379,7 +388,9 @@ main (int argc, char *argv[])
 	    x_lisp_keyword (optarg);
 	    x_elisp_keyword (optarg);
 	    x_librep_keyword (optarg);
+	    x_scheme_keyword (optarg);
 	    x_java_keyword (optarg);
+	    x_csharp_keyword (optarg);
 	    x_awk_keyword (optarg);
 	    x_tcl_keyword (optarg);
 	    x_perl_keyword (optarg);
@@ -493,7 +504,7 @@ main (int argc, char *argv[])
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 "),
-	      "1995-1998, 2000-2003");
+	      "1995-1998, 2000-2005");
       printf (_("Written by %s.\n"), "Ulrich Drepper");
       exit (EXIT_SUCCESS);
     }
@@ -615,6 +626,8 @@ This version was built without iconv()."),
       extractor_ty po_extractor = { extract_po, NULL, NULL, NULL };
 
       extract_from_file (file_name, po_extractor, mdlp);
+      if (!is_ascii_msgdomain_list (mdlp))
+	mdlp = iconv_msgdomain_list (mdlp, "UTF-8", file_name);
 
       dir_list_restore (saved_directory_list);
     }
@@ -746,8 +759,8 @@ Choice of input file language:\n"));
       printf (_("\
   -L, --language=NAME         recognise the specified language\n\
                                 (C, C++, ObjectiveC, PO, Shell, Python, Lisp,\n\
-                                EmacsLisp, librep, Smalltalk, Java,\n\
-                                JavaProperties, awk, YCP, Tcl, Perl, PHP,\n\
+                                EmacsLisp, librep, Scheme, Smalltalk, Java,\n\
+                                JavaProperties, C#, awk, YCP, Tcl, Perl, PHP,\n\
                                 GCC-source, NXStringTable, RST, Glade)\n"));
       printf (_("\
   -C, --c++                   shorthand for --language=C++\n"));
@@ -778,22 +791,22 @@ Language specific options:\n"));
   -a, --extract-all           extract all strings\n"));
       printf (_("\
                                 (only languages C, C++, ObjectiveC, Shell,\n\
-                                Python, Lisp, EmacsLisp, librep, Java, awk,\n\
-                                Tcl, Perl, PHP, GCC-source, Glade)\n"));
+                                Python, Lisp, EmacsLisp, librep, Scheme, Java,\n\
+                                C#, awk, Tcl, Perl, PHP, GCC-source, Glade)\n"));
       printf (_("\
   -k, --keyword[=WORD]        additional keyword to be looked for (without\n\
                               WORD means not to use default keywords)\n"));
       printf (_("\
                                 (only languages C, C++, ObjectiveC, Shell,\n\
-                                Python, Lisp, EmacsLisp, librep, Java, awk,\n\
-                                Tcl, Perl, PHP, GCC-source, Glade)\n"));
+                                Python, Lisp, EmacsLisp, librep, Scheme, Java,\n\
+                                C#, awk, Tcl, Perl, PHP, GCC-source, Glade)\n"));
       printf (_("\
       --flag=WORD:ARG:FLAG    additional flag for strings inside the argument\n\
                               number ARG of keyword WORD\n"));
       printf (_("\
                                 (only languages C, C++, ObjectiveC, Shell,\n\
-                                Python, Lisp, EmacsLisp, librep, Java, awk,\n\
-                                YCP, Tcl, Perl, PHP, GCC-source)\n"));
+                                Python, Lisp, EmacsLisp, librep, Scheme, Java,\n\
+                                C#, awk, YCP, Tcl, Perl, PHP, GCC-source)\n"));
       printf (_("\
   -T, --trigraphs             understand ANSI C trigraphs for input\n"));
       printf (_("\
@@ -1101,11 +1114,13 @@ flag_context_list_table_insert (flag_context_list_table_ty *table,
 				const char *name_start, const char *name_end,
 				int argnum, enum is_format value, bool pass)
 {
+  char *allocated_name = NULL;
+
   if (table == &flag_table_lisp)
     {
       /* Convert NAME to upper case.  */
       size_t name_len = name_end - name_start;
-      char *name = (char *) alloca (name_len);
+      char *name = allocated_name = (char *) xallocsa (name_len);
       size_t i;
 
       for (i = 0; i < name_len; i++)
@@ -1232,6 +1247,9 @@ flag_context_list_table_insert (flag_context_list_table_ty *table,
 	  }
       }
   }
+
+  if (allocated_name != NULL)
+    freesa (allocated_name);
 }
 
 
@@ -1370,6 +1388,11 @@ xgettext_record_flag (const char *optionstring)
 						    name_start, name_end,
 						    argnum, value, pass);
 		    break;
+		  case format_scheme:
+		    flag_context_list_table_insert (&flag_table_scheme, 0,
+						    name_start, name_end,
+						    argnum, value, pass);
+		    break;
 		  case format_smalltalk:
 		    break;
 		  case format_java:
@@ -1377,6 +1400,11 @@ xgettext_record_flag (const char *optionstring)
 						    name_start, name_end,
 						    argnum, value, pass);
 		    break;
+		  case format_csharp:
+		    flag_context_list_table_insert (&flag_table_csharp, 0,
+						    name_start, name_end,
+						    argnum, value, pass);
+		    break;
 		  case format_awk:
 		    flag_context_list_table_insert (&flag_table_awk, 0,
 						    name_start, name_end,
@@ -1466,6 +1494,58 @@ xgettext_comment_reset ()
 }
 
 
+refcounted_string_list_ty *savable_comment;
+
+void
+savable_comment_add (const char *str)
+{
+  if (savable_comment == NULL)
+    {
+      savable_comment =
+	(refcounted_string_list_ty *) xmalloc (sizeof (*savable_comment));
+      savable_comment->refcount = 1;
+      string_list_init (&savable_comment->contents);
+    }
+  else if (savable_comment->refcount > 1)
+    {
+      /* Unshare the list by making copies.  */
+      struct string_list_ty *oldcontents;
+      size_t i;
+
+      savable_comment->refcount--;
+      oldcontents = &savable_comment->contents;
+
+      savable_comment =
+	(refcounted_string_list_ty *) xmalloc (sizeof (*savable_comment));
+      savable_comment->refcount = 1;
+      string_list_init (&savable_comment->contents);
+      for (i = 0; i < oldcontents->nitems; i++)
+	string_list_append (&savable_comment->contents, oldcontents->item[i]);
+    }
+  string_list_append (&savable_comment->contents, str);
+}
+
+void
+savable_comment_reset ()
+{
+  drop_reference (savable_comment);
+  savable_comment = NULL;
+}
+
+void
+savable_comment_to_xgettext_comment (refcounted_string_list_ty *rslp)
+{
+  xgettext_comment_reset ();
+  if (rslp != NULL)
+    {
+      size_t i;
+
+      for (i = 0; i < rslp->contents.nitems; i++)
+	xgettext_comment_add (rslp->contents.item[i]);
+    }
+}
+
+
 
 static FILE *
 xgettext_open (const char *fn,
@@ -1632,7 +1712,7 @@ set_format_flags_from_context (enum is_format is_format[NFORMATS],
 	  {
 	    struct formatstring_parser *parser = formatstring_parsers[i];
 	    char *invalid_reason = NULL;
-	    void *descr = parser->parse (string, &invalid_reason);
+	    void *descr = parser->parse (string, false, &invalid_reason);
 
 	    if (descr != NULL)
 	      parser->free (descr);
@@ -1853,7 +1933,7 @@ meta information, not the empty string.\n")));
 	{
 	  struct formatstring_parser *parser = formatstring_parsers[i];
 	  char *invalid_reason = NULL;
-	  void *descr = parser->parse (mp->msgid, &invalid_reason);
+	  void *descr = parser->parse (mp->msgid, false, &invalid_reason);
 
 	  if (descr != NULL)
 	    {
@@ -1953,7 +2033,8 @@ remember_a_message_plural (message_ty *mp, char *string,
 	  {
 	    struct formatstring_parser *parser = formatstring_parsers[i];
 	    char *invalid_reason = NULL;
-	    void *descr = parser->parse (mp->msgid_plural, &invalid_reason);
+	    void *descr =
+	      parser->parse (mp->msgid_plural, false, &invalid_reason);
 
 	    if (descr != NULL)
 	      {
@@ -2140,9 +2221,11 @@ language_to_extractor (const char *name)
     SCANNERS_LISP
     SCANNERS_ELISP
     SCANNERS_LIBREP
+    SCANNERS_SCHEME
     SCANNERS_SMALLTALK
     SCANNERS_JAVA
     SCANNERS_PROPERTIES
+    SCANNERS_CSHARP
     SCANNERS_AWK
     SCANNERS_YCP
     SCANNERS_TCL
@@ -2151,14 +2234,14 @@ language_to_extractor (const char *name)
     SCANNERS_STRINGTABLE
     SCANNERS_RST
     SCANNERS_GLADE
-    /* Here will follow more languages and their scanners: pike, C#, etc...
+    /* Here may follow more languages and their scanners: pike, etc...
        Make sure new scanners honor the --exclude-file option.  */
   };
 
   table_ty *tp;
 
   for (tp = table; tp < ENDOF(table); ++tp)
-    if (strcasecmp (name, tp->name) == 0)
+    if (c_strcasecmp (name, tp->name) == 0)
       {
 	extractor_ty result;
 
@@ -2204,9 +2287,11 @@ extension_to_language (const char *extension)
     EXTENSIONS_LISP
     EXTENSIONS_ELISP
     EXTENSIONS_LIBREP
+    EXTENSIONS_SCHEME
     EXTENSIONS_SMALLTALK
     EXTENSIONS_JAVA
     EXTENSIONS_PROPERTIES
+    EXTENSIONS_CSHARP
     EXTENSIONS_AWK
     EXTENSIONS_YCP
     EXTENSIONS_TCL
@@ -2215,7 +2300,7 @@ extension_to_language (const char *extension)
     EXTENSIONS_STRINGTABLE
     EXTENSIONS_RST
     EXTENSIONS_GLADE
-    /* Here will follow more file extensions: cs ... */
+    /* Here may follow more file extensions... */
   };
 
   table_ty *tp;
